<?php declare(strict_types=1);

namespace App\Service\Api;

use App\Service\ConfigurationService;
use Psr\Log\LoggerInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

class ApiService
{
    public const URL_SCHEMA = 'http://';
    public const URL_WS_IP_ADDRESS = '127.0.0.1';
    private const URL_API_DASHBOARD = '/g/dasb';
    private const URL_API_LIST_TRIGGERED_ALERT_DEVICES = '/g/altrd';
    //get list of past devices for which the alert was triggered.
    private const URL_API_LIST_PAST_ALERT_DEVICES = '/g/alpad';
    private const URL_API_CLEAR_COUNTER_PAST_ALERTS = '/g/clral';
    private const URL_API_SYNC = '/g/synch';
    private const URL_API_DEVICE_DELETE = '/g/devde';

    public const HTTP_CLIENT_REQUEST_TIMEOUT = 1;

    protected HttpClientInterface $httpClient;

    protected ConfigurationService $configurationService;

    protected LoggerInterface $logger;

    protected string $baseUrl;

    public function __construct(HttpClientInterface $httpClient, ConfigurationService $configurationService,
                                LoggerInterface $logger)
    {
        $this->configurationService = $configurationService;
        $this->httpClient = $httpClient;
        $this->logger = $logger;

        $localPort = $this->configurationService->getServerLocalPort();

        $this->baseUrl = self::URL_SCHEMA.self::URL_WS_IP_ADDRESS.':'.$localPort;
    }

    /**
     * @return object|false
     */
    public function getDashboardData(){

        return $this->getApiCall($this->baseUrl.self::URL_API_DASHBOARD);

    }

    /**
     * @param int $alertId
     * @return object|false
     */
    public function getTrigerredAlertDevices(int $alertId) {

        return $this->getApiCall($this->baseUrl.self::URL_API_LIST_TRIGGERED_ALERT_DEVICES . '?alid=' . $alertId);

    }

    /**
     * @param int $alertId
     * @return false|mixed
     */
    public function getPastAlertDevices(int $alertId) {

        return $this->getApiCall($this->baseUrl.self::URL_API_LIST_PAST_ALERT_DEVICES . '?alid=' . $alertId);

    }

    /**
     * @param int $alertId
     * @return mixed|null
     */
    public function clearPastAlertCounter(int $alertId){

        //Return always false because of invalid response
        return $this->getApiCall($this->baseUrl.self::URL_API_CLEAR_COUNTER_PAST_ALERTS . '?alid=' . $alertId);

    }

    /**
     * @return false|mixed
     */
    public function syncCall(){

        return $this->getApiCall($this->baseUrl.self::URL_API_SYNC);

    }

    /**
     * @param int $deviceId
     * @return mixed|null
     */
    public function deleteDevice(int $deviceId){

        return $this->getApiCall($this->baseUrl.self::URL_API_DEVICE_DELETE . '?devid=' . $deviceId);

    }

    /**
     * @param string $url
     * @return mixed|false
     */
    protected function getApiCall(string $url){

        $this->logger->info('Called:' . $url, ['Api']);

        try{

            $response = $this->httpClient->request('GET', $url, [
                'timeout' => self::HTTP_CLIENT_REQUEST_TIMEOUT,
            ]);

            $data = json_decode($response->getContent());

        }catch (\Exception $exception){
            
            $this->logger->error($exception->getMessage(), ['Api']);
            return false;

        }

        return $data;

    }

}
